; Fuchs-RX-V1_0.asm, subroutinen.asm, interrupt.asm, data.ams, macro.asm
;
; ********************************************************************************************
; * Empfngersteuerung basierend auf DF1FOs Atmel-Programm V3.3
; ********************************************************************************************
;
;    **************************
;    * 2m Fuchsjagd Empfnger *
;    **************************
;
; Code fr Fuchsjagd-Empfnger-Steuerung
;
; Untersttzt den 2m-Fuchsjagdempfnger Lvl. 3.3 auf Platine 'fjrx233.brd'
;     oder der Langversion 'fjrx234.brd'
;
; Autor: Nick Roethe, DF1FO, DF1FO at darc.de
;
; Created 1.7.2004 as FJRX.ASM
;     Version   0.0: Code-Import von DFNT (LCD-Treiber), Panorama (Drehgeber)
;      3.7.04   0.1: LCD-Treiber, Timer-Interrupt,Taster/Drehgeber ok 
;      5.7.04   0.2: Code-Import I2C-Treiber von Wecker, angepasst, ok
;     29.7.04	0.3: Von 2313 auf 4433 umgestellt
;      7.8.04	0.4: Frequenzen Eingeben, Anzeigen, an Synthesizer bergeben 
;     15.8.04	0.5: Menuesystem angefangen
;     16.8.04   0.6: Menuesystem  
;     17.8.04  	0.7: Menuesystem komplett ausser Entfernung
;     18.8.04	0.8: Verbesserungen Analog-Umsetzung und -Anzeige
;     19.8.04   0.9: Entfernungsanzeige und EEPROM, = Erste Full Function Version 	
;     13.9.04	1.0: Entfernungsanzeige korrigiert, Anzeige-Mode-Umsch.gendert 
;     20.9.04	1.1: 2 RX-Frequenzen, Fix fr falsches Timer Reset 
;     28.10.04  1.2: Digitales S-Meter 
;
;      6.11.04	1.3: Umstellung auf ATMEGA 8	 		 		      
;      8.11.04	1.4: Abschwchereinstellung ber Timer 2 PWM incl. Kalibrierung
;     10.11.04	1.5: Akustisches S-Meter ber Timer 1
;     12.11.04	1.6: Verbesserungen Akust. S-Meter	
;     13.11.04	1.7: Akust. S-Meter Ein/Aus, Clean-Up
;     19.11.04	3.0: Full Function 5 Volt RX
;     16.12.04	3.1: Frequenzanpassung FFINE im Cal-Dialog
;			Version 3.1 ff untersttzt wieder RX1 und RX2
;     16.12.04  3.2: Frq.-Cal. vor Abschw.-Cal., Cal abbrechen bei RX-Off
;      7. 4.05  3.3: S-Meter sttigt bei Max statt Wrap auf 0
;     15. 8.05  3.4: Stopuhranlauf bei Kaltstart, P-On Glitch Drehgeber
;			unterdrcken, 'Save to EEPROM' anzeigen
;     30. 8.05  3.5: EEPROM-Init-Werte auf Actuals RX Brigitte
;     --------------
;      6. 1.06  0.0: Neue Hardware Lvl 3.3: Platine + div. Entfeinerungen
;		     Dazu Code FJRX 3.5 kopiert nach FJRX2 0.0 	 	
;      6. 1.06  0.1: Genderte I/O-Pins, Quarz-Frequenz angepasst  		
;      7. 1.06  0.2: Ansteuerung Vorstufe Ein/Aus, NF Degate eingebaut
;     12. 1.06  0.3: Anpassungen an erste HW: LCD-Timing, Fuses 
;     15. 1.06  0.4: Frequenz > PLL nur nach nderung (=> weniger Noise)
;     16. 1.06  0.5: Neues Mensystem im Set-Up Mode, noch nicht komplett
;     17. 1.06  0.6: Vervollstndigungen SetUp-Men
;     19. 1.06  0.7: EEPROM-Save getrennt fr Frequenzen und HW-SetUp
;     23. 1.06  0.8: Neue Kennlinie Akust. S-Meter, kleine Men-nderungen
;     30. 1.06  0.9: Fix Abschwcherzwischenwert 65dB, Menergnzungen
;      2. 2.06  1.0: Verbesserungen 'Cal Att', Aktuelle Defaultwerte Abschw.
;      4. 2.06  1.1: Automatische Abschwchung bei bersteuerung
;      5. 2.06  1.2: Abschwcherwert 65 dB: Extrapolation verbessert
;      6. 2.06  1.3: UBatt>9,9V anzeigen, Entfernungsschtzung 30dB/10x > 100m
;      3. 4.06  1.4: Fix fr zu langsame Timer/Uhr, Kennlinie ACSM steiler
;      5. 5.06	1.5: Neue Funktionen EOT-Warnung, 1..4 Sender, Schnelltaster, 
;		     EEPROM-Layout geaendert, Default Abschw fr Siemens TCA440
;      5. 5.06	1.6: Clean-Up zu 1.5, braucht aber immer noch mehr Test!	
;     22. 5.06  1.7: Neues Abgleich-Men, Texte deutscher, EEPROM-Save 3 Zonen
;     23. 5.06  1.8: Auto-Abschw. Ein/Aus, SOUND und FLASH Mechanismen,
;		     Kennlinie ACSM flacher, Low Bat Warnung, diverse CleanUps   	 	
;     24. 5.06	1.9: Frq-Einst wird Menpkt, dazu Schalterverdrahtung anders,
;		     div. Verbesserungen zu 1.8
;     24. 5.06  2.0: 2 Fixes fr CalAbs, Men beginnt mit Overview, dann Frq.	
;     26. 5.06  2.1: Absw.-Taster lang lnger, Auto-Absw-Ton deutlicher,
;		     Geschtzte Entf. +1, Fix fr Tmr 0 slow Int. Hdlr. response   
;     23. 7.06  2.2: Abschwcher-Automatik-Schwelle = S-Meter-Vollausschlag
;                    Menpunkt Batteriespannung weg
;                    Neuer Menpunkt 'Akust. S-Meter Schwelle' und Logik dazu
;		     Frequenzwechsel durch Drcken+Drehen ausgebaut 
;		     Fixes: - Brownout-Erkennung eingeschaltet (2 Fuses)
;			    - FFINE-Abgleich: -0 weg
;			    - UBat Anzeige korrekt bei Kippschalter auf
;			        Men vor Einschalten
;		     EEPROM-Layout gendert! (Valid-Code $A7) 
;     18. 8.06  2.3: Vorstufe Aus: PC0 HZ statt 1 (Fix RX Aus Erkennung > 60dB) 			      
;     22. 8.06  2.4: Frequenzwechsel durch Drcken+Drehen wieder eingebaut
;		     Einstellung Akustisches-S-Meter-Schwelle disabeled  
;		     Keine Entfernungsanzeige ber 1,5 km (stattdessen dB)
;		     Entfernungsschtzung eine Stufe nher
;     23. 8.06  2.5: Fix fr schnelles Drehen Drehgeber
;		     Neue Sounds Kanalwechsel 
;      8. 9.06  2.6: Akust. SM mit Schwelle: enabled und 2 sec Nachlauf
;     12. 9.06  2.7: Akust. SM: Nachlauf auf 3 sec, max Schwelle 3/8
;		     Frequenzbereich erweitert bis 148 MHz 
;     20. 9.06  2.8: Frequenzbereich schaltbar ..146/..148
;		     Batteriewarnungsschwelle einstellbar 5,8 - 8,0 V
;		     EEPROM-Layout gendert! (Valid-Code $A8) 
;      9.10.06  2.9: Fix fr Abstimmbereich Untergrenze bei 144-148 	
;     19.10.06  3.0: Fix fr Timer 20ms/Minute (=1,2s/h) zu langsam	
;     30.12.06  3.1: Foxoring-Erweiterungen: PFuchs bis 0,1mW, 
;		     Bei NFuchs=1 Anzeige Stopuhr statt Fuchstimer
;		     Bei PFuchs <10mW schaltet Abschw.Taster direkt auf 0dB      
;     31.12.06	3.2: Fix fr falsche Stopuhranzeige bei NFuchs=1
;	             Bei RX Aus, NFuchs=1 Anzeige Stopuhr statt Timer
;		     Bei NFuchs=1 Timer Alarm unterdrcken
;      6. 1.07	3.3: Pfuchs 1 W (fr DF7XU's Foxoring-Sender) bis 30W
;
.equ	VERSHI= '3'
.equ	VERSLO= '3'
;
; >> FUSES << :	EESAVE = 0
; ===========	CKSEL  = 1111, SUT = 11, BODEN  = 0, BODLVL = 0   ; Brown-Out Erkennung @ 4V	
;		Ergibt Hex-Pattern D13F
;
;
;
; -------------------------------------------------------------------------------------------
; nderungen in dieser Version:   Thomas Schiller, DC7GB@vfdb.net
; -------------------------------------------------------------------------------------------
;
;
; V3.3+	Diese Version hat folgende nderungen zur Originalfassung "V. 3.3" von DF1FO:
;
;	- Anpassung auf AVR-Studio 4
;	- Aufteilung in 5 Dateien (Fuchs-RX-V1, subroutinen, interrupt, data, macro).
;	- Optik des Codes an einigen STellen verbessert, um die Lesbarkeit zu erhhen.
;	- Codebereiche zusammen gefasst und Speicherplatz optimiert. Inbesondere
;	  einige Unterprogramme umgeschrieben und ergnzt, sowie ab MAIN07 die
;	  Anzeigekette mit vielen Macro-Aufrufen auf ein Tabellen-Lookup umgestellt.
;	- Nach dem Einschalten wird die Versionsnummer und ein "+" ausgegeben.
;	- Es knnen nun 6 Frequenzen vorprogrammiert werden.
;	- Fr jede Frequenz kann nun eine eigene Sendeleistung vorgegeben werden.
;	- Die Sonderzeichen der Kanalanzeige sind jetzt kleine rmische Ziffern.
;	- Die akustische Frequenzsignalisierung gibt die Kanalnummer in CW aus.
;	- Die Drehrichtung des Abschwchers wurde verndert: nach links == hhere Dmpfung.
;	- Fehler in EEPROM Voreinstellung behoben (ACSLM vertauscht).
;	- Die EEPROM-Standardwerte sind auf meine gemessenen Werte voreingestellt.
;	- EEPROM-Bytes werden nur dann geschrieben, wenn sie sich von den schon gespeicherten
;	  Werten im EEPROM unterscheiden.
;	- Jede EEPROM-Schreib- oder Leseoperation wird akustisch in CW angezeigt (E W oder E R).
;	- Einige Anzeigetexte wurden geendert ("QRG" statt "Freq.", "Timer" statt "Fuchs").
;
;
; Verbrauchter Speicherplatz:
;
;	Version 3.3:	Used: 7666 Bytes = 93,6%
;	Version 3.3+:	Used: 7992 Bytes = 97,6%

.include 	"m8def.inc"	; System fr Atmels ATmega8
.listmac			; Macro Expansion Listing On
.include	"macro.asm"

; ********************************************************************************************
; Variablen:

.dseg
.org	SRAM_START	; Start of RAM address range ($0060)

; ++ Eingestellte Parameter, werden im EEPROM gerettet

EEPCLR:	.BYTE	1	; $FF wenn EEPROM gelscht, $A8 wenn gltig

EEP0:			; Beginn des Frequenz-Bereichs
FRQ1:	.BYTE	2	; Frequenz-Codewert Frequenz 1
			;      0..3360 = 143,9..148,1, LSB,MSB
FRQ2:	.BYTE	2	; Frequenz-Codewort Frequenz 2 
FRQ3:	.BYTE	2	; Frequenz-Codewort Frequenz 3
FRQ4:	.BYTE	2	; Frequenz-Codewort Frequenz 4
FRQ5:	.byte	2	; Frequenz-Codewort Frequenz 5
FRQ6:	.byte	2	; Frequenz-Codewort Frequenz 6
FRQX:	.BYTE	1	; Aktive Frequenz 1..6

; Grenze zwischen Frequenz- und HW-SetUp-Bereich

EEP1:
NFOX:	.BYTE	2	; Zahl der Fchse Z,E  1..10	
TFOXS:	.BYTE 	2	; Fuchs-Laufzeit Sekunden Z,E  1..99
TFOXMS: .BYTE   2       ; Fuchs-Laufzeit Milli-Sekunden H,Z 0..98 in 2er Schritten
PFOX1:	.byte	1	; Sendeleistung Fuchs 1
PFOX2:	.byte	1	; Sendeleistung Fuchs 2
PFOX3:	.byte	1	; Sendeleistung Fuchs 3
PFOX4:	.byte	1	; Sendeleistung Fuchs 4
PFOX5:	.byte	1	; Sendeleistung Fuchs 5
PFOX6:	.byte	1	; Sendeleistung Fuchs 6
PFOXBN:	.BYTE	1	; Sendeleistung der aktiven Frequenz in FRQX:
			;    Coded 0=1W, 1=3W, 2=10W ..15=30W  
NFREQ:	.BYTE	1	; Zahl der Frequ. 1..6
TALARM:	.BYTE	2	; EOT Alarm Vorlaufzeit 0..31 sec, 0 = Aus Z,E
ACSMLV: .BYTE	1	; Akustisches S-Meter Schwelle 0..7 Achtel

; Grenze zwischen HW-SetUp und Abgleich-Bereich

EEP2:
ABSTAB:	.BYTE	13	; Tabelle Abschwcher-Codes 0,10,20..120 dB
FFINE:	.BYTE	1	; Frequenz Fein 64 -/+ 25 = -25..+25 * 1,25 kHz
UBFINE:	.BYTE   1	; Skalierungswert fr Ub-Messung 0..255
FBER68:	.BYTE 	1	; Frequenzbereich 0:144..146, 1:144..148
UBWBIN: .BYTE   1       ; Schwelle Batteriewarnung in 1/10 V (60..80)
EEPEND:			; Ende der geretteten Einstellungen

; ++ Zustands-Variablen

MODE:	.BYTE	1	; Mode (Beschreibung siehe Texte ganz unten) 
			; 0 = RX aus, 2 = Frequenzeingabe, 3 = Peilen
			; 4..x = Hauptmen
			; $10..$1x = Untermen Einstellungen
			; $20..$2x = Untermen Abgleich
FRQBIN:	.BYTE	2	; Aktuelle Frequenz 0..3360 = 143,9..148,1, LSB,MSB
FRQCHG: .BYTE   1	; 1 = Frequenz gendert => PLL neu laden
FRQEEP:	.BYTE	1	; 1 = 'Genderte Frequenzen mssen ins EEPROM'-Flag	
SUEEP:	.BYTE	1	; 1 = 'Genderte SetUp-Param. mssen ins EEPROM'-Flag
AUTOON:	.BYTE	1	; Abschwcher-Automatik ein = ' ', aus = '*'
ACSMON:	.BYTE	1	; Akustisches S-Meter ein = $40, aus = $00
ACSTIM:	.BYTE	1	; Akust.S.-Meter > Schwelle, Nachlaufzeit (*100ms)
FRQ:	.BYTE	4	; Frequenz-Dezimalstellen 1M,100k,10k,1k
FRQXSY: .BYTE	1	; Symbol fr Frequenz 1..6 (kleine rmische Zahlen) 
STOPW:	.BYTE	5	; Stopuhr Eh,Zm,Em,Zs,Es
STOPWR:	.BYTE	1	; 1 = Stopuhr luft, 0 = steht
TIMER:	.BYTE	4	; Count Down Timer Zs,Es,Hms,Zms zhlt in 20 ms Schritten
FOX:	.BYTE	2	; Current Fox Z,E  1..16
ABSBIN:	.BYTE	1	; Abschwcher Binaer-Wert 0..24 fr 0, 5, 10..120 dB
ABSCHW:	.BYTE	3	; Abschwcher Anzeige 0..120 dB in 5 dB Stufen H,Z,E
DIST:	.BYTE	4	; Distanz-Anzeige H,Z,E
ABSVAL:	.BYTE	3	; Abschwcher Analogwert 0..255 H,Z,E
FFIVAL: .BYTE   3	; FFINE Anzeigewert -/+0..25  VZ,Z,E  
UBFVAL: .BYTE   3	; UBat Korrektur UBFINE Anzeigewert 0..255 H,Z,E
PFOX:	.BYTE	4	; Sendeleistung Watt Anzeigewert Z,E,h,z			
AAWAIT:	.BYTE	1	; Wartezeit nach 5dB-Auto-Abschwchung (n * 100 ms)
FLATIM:	.BYTE	1	; Verbleibende Flashtime fr Text-Flash (n * 100 ms)
SOUPNT:	.BYTE	2	; Pointer auf Soundstring LSB,MSB; Fertig bei MSB = 0
UBWARN:	.BYTE   2	; Batteriewarnungsschwelle V/mV E,h 
EEWRAK:	.byte	1	; 1 = mindestens 1 Byte wurde ins EEPROM geschrieben (low level Flag)

; ++ Analog-Werte

UBATT:	.BYTE	3	; Batterie-Spannung in V/mV    Z,E,h 		
UMESSA:	.BYTE	1	; Messspannung fr akustisches S-Meter
UMESSP:	.BYTE	4	; Spitzenwertspeicher Umess, ltester Wert in *+3 
UMESS:	.BYTE	8	; Messspannung Bargraph 8 Stellen

;++ Display-Ansteuerung

LCDDAT:	.BYTE   16	; Data to be displayed in LCD

; ++ Kommunikation mit Int.-Handler

SWIFLG: .BYTE   1       ; Switch Flags, Bit 0=Up, 1=Down, 2=Up+Pressed, 
			; 3=Down+Pressed, 4=Drehdrcker Puls
			; 5=Schnell-Taster Kurz, 6=Schnell-Taster Lang 
TICK:	.BYTE	1	; 100 ms Timing Flag 
ALATIM:	.BYTE	1	; EOT Alarm Flag, 1 = Alarm auslsen 
BATTIM:	.BYTE	1	; Batterie-Prf-Zeit, 1 = prfen 

; ++ Interrupt-Handler

DEBP1:	.BYTE   1	; Debounce Counter Drehgeber 
DEBDD:	.BYTE   1	; Debounce Drcker Drehgeber 
DEBST:	.BYTE   1	; Debounce Schnell-Taster 
TIK3U1:	.BYTE   1       ; Zaehlt alle 320 us bis 3 (1 msec)
TIK3U2:	.BYTE   1       ; Zaehlt alle 320 us bis 25 (8 msec)
TIK8M:	.BYTE	1	; Zaehlt jede 8 msec bis 5
TIK20M: .BYTE	1	; Zaehlt jede 20 msec bis 50 (1 sec)	
TIK100: .BYTE   1	; Zaehlt jede 20msec bis 5 (100 ms) 
TIK1S:	.BYTE	1	; Zaehlt jede 1 s bis 60 (1 min)



.cseg
.org	0

; ++++ Interrupt Vectors

	RJMP	START           ;  0 Reset   
        RJMP    START           ;  1 Ext Int 0 
        RJMP    START           ;  2 Ext Int 1 
        RJMP    START           ;  3 Timer Compare 2    
        RJMP    START           ;  4 Timer Overflow 2 
        RJMP    START           ;  5 Input Capture 1 
        RJMP    START           ;  6 Timer Compare 1A
        RJMP	START		;  7 Timer Compare 1B
        RJMP    START           ;  8 Timer Overflow 1
        RJMP    TIMINT          ;  9 Timer Overflow 0
        RJMP    START           ; $A SPI     
        RJMP	START		; $B USART Receive Complete
        RJMP	START		; $C USART Data Reg. Empty
        RJMP    START           ; $D USART Transmit Complete
        RJMP    START           ; $E A/D-Converter
        RJMP    START           ; $F EEPROM
        RJMP    START           ;$10 Analog Comparator
        RJMP    START           ;$11 2-wire I/F
        RJMP    START           ;$12 SPM complete

; ++++	Entry-Point:

START:  ldi     R16,LOW(RAMEND)		; Stackpointer
        out     SPL,R16			; ...an das
        ldi     R16,HIGH(RAMEND)	; ...RAM-Ende
        out     SPH,R16			; ...setzen

; ++++ Initialize RAM area

	clr	XH
	ldi	XL,LOW(SRAM_START)
	clr	R16
STA01:	st	X+,R16		
	cpi	XL,LOW(RAMEND+1)
	brne	STA01
	cpi	XH,HIGH(RAMEND+1)
	brne	STA01

; ++++ Initialize Port B Bit 0 as Input P1 with Pull-Up
;			     1 as Sound Output (=OC1A)
;			     2 as Sound Degate Output        
;			     3..5 used by ISP	  
;			     3 also Uregel Output (OC2)	
;			     4 also Input Drcker with Pull-Up

        ldi     R16,$0E
        out     DDRB,R16
        ldi	R16,$11	
        out	PORTB,R16

; ++++ Initialize Port C Bit 0 as Vorstufe Ein/Aus Output
;			     1,2 as Analog In Ub,Umess
;			     3 as Input P0 with Pull-Up 
;			     4 as I2C SCL Line, Output, OC, Hi      
;		     	     5 as I2C SDA Line, BiDi, OC, Hi

        ldi     R16,$01
        out     DDRC,R16
        ldi	R16,$08	
        out	PORTC,R16

; ++++ Initialize Port D Bit 0..5 as Output for Display
;	                 Bit 6,7 as Input with Pull-Up for Mode-Schalter

        ldi     R16,$3F 
        out     DDRD,R16
        ldi	R16,$C0
        out	PORTD,R16

; ++++ Initialize A/D-Converter

        LDI     R16,$01         ; Select Analog Input 1
        OUT     ADMUX,R16
        LDI     R16,$86         ; Enable ADC, Clock Divider 64 = 78 kHz
        OUT     ADCSR,R16	;     = 180 usec Conversion Time

; ++++ Intialize Timer/Ctr 0 as 320 usec Timing Interrupt (5Mhz/64/25)

	ldi	R16,$03		; Set Clock Divider to 64
	out	TCCR0,R16
	ldi	R16,-25		; Set Counter to Start-Value
	out	TCNT0,R16
	ldi	R16,$01
	out	TIMSK,R16	; Enable Counter 0 Overflow Interrupt
	;
	ldi	R25,50		; Wait a little
	rcall	DELXMS

; ++++ Intialize Timer/Counter 1 as Variable Osc. for Acoustic S-Meter
; -- CTC-Mode on OC1A Pin, f is 5,0 MHz /2 /OCR1AH_L 

	ldi 	R16,$00		; $00 = Sound Off, $40 = On	
	out	TCCR1A,R16
	ldi 	R16,$09
	out	TCCR1B,R16
	ldi 	R16,$0A		; Komparator fuer Counter Reset & Output Toggle
	out	OCR1AH,R16 
	ldi 	R16,0		; $0A00 = 1 kHz Startwert
	out	OCR1AL,R16

; ++++ Intialize Timer/Counter 2 as fast PWM (19 kHz) for Uabschw

	ldi	R16,$69		; Clock Divider 1, fast PWM, OC2 active
	out	TCCR2,R16

; ++++ Hole Parameter aus EEPROM

	rcall	GETEEP		; Parameter aus EEPROM holen
	lds	R16,EEPCLR
	cpi	R16,$A8		; hat EEPROM gltigen Inhalt?
	breq	STA03		; ja...
	rcall	EEPNEW		; EEPROM initialisieren

; ++++ Initialize some Variables	

STA03:	stsi	EEWRAK,0	; EEPROM Write-Flag lschen
	stsi	STOPWR,1	; Stopuhr luft nach Power-On an
	stsi	FRQCHG,1	; Laden der PLL anfordern 
	stsi	ABSBIN,2	; Abschwcher auf 10 dB
	stsi	AUTOON,' '	; Abschwcher-Automatik Ein
	stsi	AAWAIT,5	; Abschwchermechanismus verzgert einschalten 
	sbic	PINB,4 		; Wenn Drcker gedrckt
	rjmp	STA031		; 
	stsi	MODE,$20	; .. dann Abgleichmode

; ++++ Start Interrupt Handler

STA031:	rcall	CLRTIM		; Fuchstimer auf Startwert	
	sei			; Global Interrupt enable

; ++++ LCD initialisieren und Start-Text anzeigen

	rcall	INILCD
	flash	STRPO,20	; Start-Message fr 20x 100ms anzeigen		
	ldi	R25,100		; Wait 100 ms
	rcall	DELXMS

; ++++	Frequenz, Leistung und Batteriespannung laden:

	rcall	GETFSY		; Aktuelles Frequenzsymbol laden
	rcall	GETFRQ		; Aktuelle Frequenz laden
	rcall	GETPF		; Aktuelle Leistung laden
	rcall	CONVF		; Frequenz Dezimal ausrechnen
	rcall	CONVPF		; Leistung Dezimal ausrechnen
	rcall	CONVBW		; Batterie-Warnung Dezimal ausrechnen

; ++++	Ende der Initialiserung:

	stsi	SWIFLG,0	; Drehgeber Flags initialisieren
	sound	SOUPO		; Startsound abspielen
	rjmp	MAIN6		; Start Main Loop with A/D of UBat



; *******************************************************************************************
; Hauptscheife

MAIN:
	
; ++++ Mode Updaten

MAIN2:	lds	R16,MODE	; Mode holen
	sbis	PIND,6		; Wenn Schalter nicht auf Men (FRQ)
	rjmp	M21

; Schalter in Stellung: Peilen...

	sbis	PINC,4		; .. und RX Power Off
	ldi	R16,0		; .... dann Mode 0	 
	sbic	PINC,4		; .. und RX Power On
	ldi	R16,3		; .... dann Mode 3	 
	rjmp	M29

; Schalter in Stellung: Men

M21:	sbic	PIND,6		; Wenn Schalter auf Men (FRQ)
	rjmp	M29
	mov	R18,R16
	andi	R18,$FC		; und Mode < 4
	brne	M29
	cpi	R16,2		; aber nicht 2
	breq	M29
	ldi	R16,7		; .. dann Mode = 7	
	flash	STRMEN,30	; Start Menu Message

M29:	sts	MODE,R16	; Mode speichern

; Je nach Mode: Drehdrcker + Taster behandeln:

MAIN3:	lds	R16,MODE	; Mode holen
	cli			
	lds	R17,SWIFLG	; Switch Flags holen
	clr	R18		; .. und bis zum nchsten 
	sts	SWIFLG,R18	; ...Interrupt lschen
	sei
	tst	R17		; Drehgeber bettigt?
	breq	MAIN31		; nein...
	clr	R18		; ... ja, dann laufende Flashes abbrechen
	sts	FLATIM,R18

; Schalter in Stellung: Abschwcher

MAIN31:	sbrs	R17,6		; Lang drcken
	rjmp	M300		; nein...

; Lang gedrckt:

	lds	R18,AUTOON	; Abschwcher-Automatik toggeln
	ldi	R19,$0A		; .. ' ' <> '*'
	eor	R18,R19
	sts	AUTOON,R18	
	cpi	R18,' '		; Passenden Quittungston + Text erzeugen 
	breq	M3X1
	sound	SOUAA0
	flash	STRAA0,10
	rjmp	M309

M3X1:	sound	SOUAA1
	flash	STRAA1,10
	rjmp	M309	

M300:	sbrs	R17,5		; Kurz drcken
	rjmp	M309		; nein...

; kurz gedrckt:

	lds	R16,ABSBIN	; = Abschwcher hochschalten
	cpi	R16,9		; Wenn > 40 dB		 
	brcs	M301
	stsi	ABSBIN,8	; .. dann auf 40 dB
	rjmp	M302

M301:	stsi	ABSBIN,2	; .. sonst auf 10 dB	
M302:	lds	R18,PFOXBN	; Wenn allerdings PFOX <= 7 (3 mW)
	andi	R18,$08
	brne	M303
	stsi	ABSBIN,0	; .. dann Abschwcher immer gleich auf 0dB	
M303:	push	R17
	rcall	CONVAB		; .. neuen Abschwcherwert einstellen
	pop	R17
M309:	andi	R17,$9F		; Flag Bits 5, 6 lschen

; Bei Mode 0 (RX Off) sonst nix tun, nur Flags lschen

	lds	R16,MODE
	cpi	R16,3		; Im Mode 3...
	brne	M30
	rcall	UDABS		; ...Abschwcher ndern, ACSM an/aus, Kanal
	rjmp	M399		; fertig

M30:	cpi	R16,2		; Im Mode 2...
	brne	M31
	rcall	UDFRQ		; ...Kanal oder Frequenz ndern	(14#.### ndern)
	rjmp	M399		; fertig

M31: 	mov	R18,R16		; Im Mode 4...
	andi	R18,$FC		; ...untere Bits in r18-Kopie lschen
	brne	M331		; Mode ist >= 4
	rjmp	M399		; fertig

; Mode ist >= 4

M331:	sbrc	R17,0		; Wenn Up Flag
	inc	R16		; .. Mode erhhen
	sbrc	R17,1		; Wenn Down Flag	
	dec	R16		; .. Mode verringern

; Mode Range hier im Bereich: $4..$7 halten, wenn +/-1 ausserhalb

	cpi	R16,8		; Mode = 8?
	brne	M332		; nein...
	ldi	R16,4
M332:	cpi	R16,3		; war Mode zuvor 4?
	brne	M333		; nein...
	ldi	R16,7		; ja, dann auf 7 setzen

; Mode Range hier im Bereich: $10..$16 halten, wenn +/-1 ausserhalb

M333:	cpi	R16,$17 	; Mode =$17 (berlauf)?	
	brne	M334
	ldi	R16,$10		; ...ja, dann auf $10 setzen
M334:	cpi	R16,$0F		; Mode =$F (Unterlauf)?
	brne	M335
	ldi	R16,$16 	; ...ja, dann auf $16 setzen

; Mode Range hier im Bereich: $20..$27 halten, wenn +/-1 ausserhalb

M335:	cpi	R16,$28		; Mode =$28 (berlauf)?
	brne	M336
	ldi	R16,$20		; ...ja, dann auf $20 setzen
M336:	cpi	R16,$1F		; Mode =$1F (Unterlauf)?
	brne	M337
	ldi	R16,$27		; ...ja, dann auf $27 setzen
M337:	sts	MODE,R16	; Mode speichern

; Abfrage, was ausgefhrt werden soll:

M34:	cpi	R16,4		; Wenn Mode = 4			(Uhr Stop)
	brne	M35
	sbrs	R17,4		; .. und Drckerpuls
	rjmp	M35
	lds	R18,STOPWR	; .. und Stopuhr steht
	tst	R18
	brne	M342
	rcall	CLRSTW		; .... dann Stopuhr lschen und starten
	rjmp	M35
M342:	clr	R18		; 
	sts	STOPWR,R18	; .... sonst Stopuhr anhalten
	rjmp	M399

M35:	cpi	R16,5		; Wenn Mode = 5			(TmrStart)
	brne	M36
	rcall	UDCTMR		; .. dann Timer lschen/ndern
	rjmp	M399

M36:	cpi	R16,6		; Wenn Mode = 6			(Einstell- men)
	brne	M37
	sbrs	R17,4		; .. und Drehdrcker Click
	rjmp	M37		
	ldi	R16,$10		; .. dann neuer Mode $10
	sts	MODE,R16
	rjmp	M399

M37:	cpi	R16,7		; Wenn Mode = 7			(###.### ndern)
	brne	M310
	sbrs	R17,4		; .. und Drehdrcker Click
	rjmp	M310		
	ldi	R16,$02		; .. dann neuer Mode 2 (Frequenz ndern)
	sts	MODE,R16
	rjmp	M399

M310:	cpi	R16,$10		; Wenn Mode = $10		(N Timer)
	brne	M311
	rcall	UDNF		; .. dann N Timer verndern
	rjmp	M399

M311:	cpi	R16,$11		; Wenn Mode = $11		(T Fuchs >##<,##s)
	brne	M312
	rcall	UDTFS		; .. dann T Fuchs Sekunden verndern
	rjmp	M399

M312:	cpi	R16,$12		; Wenn Mode = $12		(T Fuchs ##,>##<s)
	brne	M313
	rcall	UDTFMS		; .. dann T Fuchs ms verndern
	rjmp	M399

M313:	cpi	R16,$13		; Wenn Mode = $13		(P QRG ##,# mW)
	brne	M314
	rcall	UDPF		; .. dann Leistung Fuchs verndern
	rjmp	M399

M314:	cpi	R16,$14		; Wenn Mode = $14		(N QRG)
	brne	M315
	rcall	UDNFRQ		; .. dann Zahl Frequenzen verndern
	rjmp	M399

M315:	cpi	R16,$15		; Wenn Mode = $15		(T Alarm)
	brne	M316
	rcall	UDTA		; .. dann EOT Alarmzeit verndern
	rjmp	M399

M316:	cpi	R16,$16		; Wenn Mode = $15		(Akust. SM))
	brne	M320
	rcall	UDACSM		; .. ACSM Schwelle verndern
	rjmp	M399

M320:	cpi	R16,$20		; Wenn Mode = $20
	brne	M321
				; .. nix tun

M321:	cpi	R16,$21		; Wenn Mode = $21
	brne	M322
	rcall	UDEENE		; .. EEPROM neu initialisierem
	rjmp	M399

M322:	cpi	R16,$22		; Wenn Mode = $22
	brne	M323
	rcall	CALUB		; .. dann U Bat kalibrieren
	rjmp	M399

M323:	cpi	R16,$23		; Wenn Mode = $23
	brne	M324
	rcall	CALFFI		; .. dann Frequenz kalibrieren
	rjmp	M399

M324:	cpi	R16,$24		; Wenn Mode = $24
	brne	M325
	sbrc	R17,4		; .. und Drehdrcker Click
	rcall	CALABS		; .. dann Frequenz/Abschwcher eichen
	rjmp	M399

M325:	cpi	R16,$25		; Wenn Mode = $25
	brne	M326
	rcall	UDCHGF		; .. dann obere Frequenzgrenze (146/148 MHz) toggeln
	rjmp	M399

M326:	cpi	R16,$26		; Wenn Mode = $26
	brne	M327
	rcall	UDUBW		; .. Batterie-Warnungs-Pegel einstellen
	rjmp	M399

M327:	cpi	R16,$27		; Wenn Mode = $27
	brne	M399
	rcall	UDEEWR		; .. dann nach Drehdrcker Klick EEPROM schreiben

; Menoperationen ab hier beendet:

M399:	rcall	ADUM		; Messspannung A/D-Wandeln und behandeln

	lds	R16,TICK	; 100ms Flag
	tst	R16		; ...testen
	brne	MAIN51		; -> 100ms abgelaufen
	rjmp	MAIN

; Ab hier alle 100ms:

MAIN51:	clr	R16		; 100ms-Flag
	sts	TICK,R16	; ...lschen

; ++++ Wenn Empfnger bersteuert und Automatik Ein:
;      Abschwcher 5 dB hher und Tat (je 100 ms 500 Hz und 1 kHz)

MAINA:	lds	R16,AUTOON	; Automatik eingeschaltet?
	cpi	R16,' '
	brne	MAINB		; .. sonst nix tun

	lds	R16,AAWAIT	; Wartezeit, weil gerade erst abgeschwcht?
	tst	R16
	breq	MAINA1		; nein
	dec	R16		; ja...
	sts	AAWAIT,R16	; ...dann nur Wartezhler dekrementieren
	rjmp	MAINB

MAINA1:	lds	R16,UMESSA	; Messpannung > Schwelle? 
	cpi	R16,130		; 128 ist S-Meter Vollausschlag
	brcc	MAINA2		
	rjmp	MAINB

MAINA2:	lds	R16,ABSBIN	; .. ja: Abschwcher hochschalten
	cpi	R16,24		; .. ausser wenn schon am Anschlag
	breq	MAINB
	inc	R16
	sts	ABSBIN,R16	
	rcall	CONVAB		; .. neuen Abschwcherwert einstellen
	stsi	AAWAIT,1	; .. Wartezeit vor nchster Abschwchung
	lds	R16,FLATIM	; .. Wenn Textanzeige nicht aktive
	tst	R16
	brne	MAINB
	lds	R16,SOUPNT+1	; .... dann wenn Soundgenerator nicht busy
	tst	R16
	brne	MAINA3
	sound	SOUABS		; ...... Zweiton erzeugen
	rjmp	MAINB

MAINA3:	sound	SOUAB2		; ...... Sonst Folge-Zweiton erzeugen	

; ++++ Alarm-Tonfolge erzeugen, wenn Flag gesetzt und Mode = 3
;      und mehr als 1 Fuchs

MAINB:	lds	R17,ALATIM	; Flag holen
	clr	R18
	sts	ALATIM,R18	; und lschen
	tst	R17		; Flag > 0?
	breq	MAINB0	
	lds	R17,MODE	; und Mode = 3
	cpi	R17,3
	brne	MAINB0
	lds	R18,NFOX+1	; und > 1 Fuchs.. 
	cpi	R18,1
	breq	MAINB0
	sound	SOUALA		; .. dann Tonsignal 
	flash	STRALA,8	; .. und Meldung zeigen

; ++++ ACSM > Schwelle Nachlaufzeit dekrementieren (falls > 0) 

MAINB0:	lds	R17,ACSTIM	; ACSM-Timer > 0
	tst	R17
	breq	MAINB1
	dec	R17		; .. dann dekrementieren
	sts	ACSTIM,R17

; ++++ Batterie checken wenn Flag BATTIM gesetzt

MAINB1:	lds	R17,BATTIM	; Flag holen
	clr	R18
	sts	BATTIM,R18	; und lschen
	tst	R17		; Flag > 0?
	breq	MAINB6	
	;
	lds	R16,UBATT	; Batteriespannung 10V-Stelle holen
	tst	R16		; Wenn > 0: Bat ok
	brne	MAINB6		
	;
	lds	R16,UBATT+1	; Sonst 1V-Stelle holen 	
	lds	R17,UBWARN
	cp	R16,R17		
	breq	MAINB3		; Wenn = Alarmschwelle: nchste Stelle
	brcs	MAINB5		; Wenn < Alarmschwelle: Alarm
	rjmp	MAINB6		; Wenn > Alarmschwelle: Bat ok 
	;
MAINB3:	lds	R16,UBATT+2	; Wenn 1V-Stelle gleich
	lds	R17,UBWARN+1	; .. 100mV-Stelle holen
	cp	R16,R17		; Wenn > Alarmschwelle: Bat ok
	brcc	MAINB6

MAINB5:	flash	STRBAT,50	; Sonst Alarm-Text und -Sound
	sound	SOUBAT
MAINB6:

; ++++ Batteriespannung A/D-konvertieren und skalieren

MAIN6:	ldi	R16,1		; Analog-Channel 1 = UBATT
	lds	R19,UBFINE	; Scaling Factor, ist $2xxd
	swap	R19	        ;  mit xx = UBFINE und d = dont care
	andi	R19,$0F
	ori	R19,$20
	lds	R18,UBFINE	; Scaling Factor, Low
	swap	R18
	rcall	ANACON
	ldi	XH,0		; Pointer auf Anzeige-Fenster
	ldi	XL,UBATT
	rcall	B3BCD		; Ergebnis in BCD wandeln und ablegen
	rcall	CONVAB		; Abschwcher-Einstellung in Entfernung umrechnen
	rcall	CONVUM		; Messspannung in Bargraph-Anzeige wandeln

; ++++   Je nach Mode Display laden

	lds	R17,FLATIM	; Flash-Timer
	tst	R17		; >0?
	breq	MAIN7		; ja, dann Anzeige erneuern
	dec	R17		; Flash-Timer dekrementieren
	sts	FLATIM,R17
	rjmp	MAIN8		; -> Display nicht ndern!

; LCD-Anzeige auf neusten Stand bringen:

MAIN7:	lds	R16,MODE
	ldi	XL,low((SHOWTAB<<1)-2)	; Anfang des Anzeigestring-Arrays
	ldi	XH,high((SHOWTAB<<1)-2)
	ldi	ZL,low(MODETAB<<1)	; Anfang des Moden-Arrays
	ldi	ZH,high(MODETAB<<1)
MTAB1:	adiw	XL,2		; X = X + 2
	lpm	r18,z+		; Byte aus Mode-Tabelle im Programmspeicher laden
	tst	r18		; Tabellenende erreicht?
	brmi	MTAB2		; ja...
	cp	r18,r16		; Bytes identisch?
	brne	MTAB1		; nein, dann weiter suchen...

; Mode in Tabelle gefunden:

MTAB2:	movw	ZH:ZL,XH:XL	; X -> Z
	lpm	XL,z+		; Anzeige-Stringadresse
	lpm	XH,z+		; ...laden
	movw	ZH:ZL,XH:XL	; X -> Z
	tst	ZH
	breq	MSOND		; Nullvektor -> Sonderbehandlung ntig
	rcall	LCDSTR		; String aufbauen
	rcall	OUTLCD		; .. und Ergebnis anzeigen	
	rjmp	MAIN8

; Sonderbehandlungen bearbeiten:

MSOND:	cpi	R16,0
	brne	M73

; Mode 0:

	lds	R18,NFOX+1	; Mode RX Aus: Wenn nur 1 Fuchs.. 
	cpi	R18,1
	breq	M701
	show	STR000		; Timer anzeigen
	rjmp	M73

M701:	show	STR001		; Stopuhr anzeigen

; Mode 3:

M73:	cpi	R16,3
	brne	M74
	lds	R18,NFOX+1	; Mode Peilen: Wenn nur 1 Fuchs.. 
	cpi	R18,1
	breq	M731
	show	STR030		; Timer anzeigen
	rjmp	M74

M731:	show	STR031		; Stopuhr anzeigen

; Mode 4:

M74:	cpi	R16,4
	brne	M725
	lds	R18,STOPWR
	tst	R18
	breq	M741
	show	STR040		; "Uhr Stop" anzeigen
	rjmp	M725

M741:	show	STR041		; "UhrStart" anzeigen

; Mode 25:

M725:	cpi	R16,$25
	brne	MAIN8
	lds	R18,FBER68	; Gewhlten Abstimmbereich zeigen
	tst	R18
	brne	M7251
	show	STR250		; "FBereich 144-146" anzeigen
	rjmp	MAIN8

M7251:	show	STR251		; "FBereich 144-148" anzeigen	

; ++++   Falls RX Power On und Frequenz gendert: Synthesizer laden

MAIN8:	sbis	PINC,4		; RX Power On Flag
	rjmp	MAIN85		; RX ist aus!
	lds	R16,FRQCHG	; Change Frequency Flag gesetzt?
	tst	R16
	breq	MAIN89
	rcall	SETPLL		; Ja, Frequenz in PLL laden
	stsi	FRQCHG,0	; .. und Flag lschen
	rjmp	MAIN89

MAIN85:	stsi	FRQCHG,1	; RX Off, Change Frq Flag setzen	
MAIN89:

; ++++ Bei Mode 0 oder 3 und FRQEEP: EEPROM schreiben

MAIN9:	lds	R16,MODE	; Wenn Mode 0 oder 3
	cpi	R16,0
	breq	MAIN91
	cpi	R16,3
	brne	MAIN99	

MAIN91:	lds	R16,FRQEEP	; 'Save Frq'-Flag
	tst	R16		; ...gesetzt?
	breq	MAIN92		; nein...
	rcall	SAVFRQ		; Frequenzen in EEPROM retten
	rjmp	MAIN93

MAIN92:	lds	R16,SUEEP	; 'SetUp'-Flag
	tst	R16		; gesetzt?
	breq	MAIN99		; nein...
	rcall	SAVSU		; Parameter in EEPROM retten

; Testen, ob EEPROM tatschlich beschrieben wurde:

MAIN93:	lds	r16,EEWRAK	; Byte-Write-Flag gesetzt?
	tst	r16
	breq	MAIN99		; nein...
	stsi	EEWRAK,0	; ja, dann Flag lschen und
	flash	STRSAV,10	; EEPROM-Anzeige ausgeben
	sound	SOUEW		; EEPROM Schreiben signalisieren

; Soundgenerator abarbeiten

MAIN99:	rcall	SOUGEN
	rjmp	MAIN



.include	"subroutinen.asm"
.include	"interrupt.asm"
.include	"data.asm"


